import aiohttp
import traceback
from nonebot.log import logger
from .Config import config_parser


class Search:
    def __init__(self, plain):
        self.plain = plain

    async def get_search(self) -> str:
        url = "https://api.tavily.com/search"
        headers = {
            "Content-Type": "application/json",
            "Authorization": config_parser.get_config("search_api"),
        }
        data = {
            "query": self.plain,
            "include_answer": True,
        }

        try:
            async with aiohttp.ClientSession() as session:
                async with session.post(
                    url, headers=headers, json=data, ssl=False
                ) as response:
                    response_data = await response.json()
                    if response_data["answer"]:
                        return response_data["answer"]
                    else:
                        return False  # 没有相关内容
        except Exception:
            logger.warning(traceback.format_exc())
            return None  # 错误



from nonebot.plugin.on import on_message, on_notice, on_command, on_fullmatch
from nonebot.plugin import PluginMetadata, require
from nonebot.rule import to_me
from nonebot.permission import SUPERUSER
from nonebot.params import CommandArg
import asyncio
from nonebot.adapters.onebot.v11 import (
    GROUP,
    Message,
    MessageEvent,
    GroupMessageEvent,
    PokeNotifyEvent,
    Bot,
)
import random
require("nonebot_plugin_localstore")
from .utils import (
    hello__reply,
    poke__reply,
    format_message,
)
from collections import defaultdict

from . import moe_llm as llm
from .ModelSelector import model_selector
from .TemperamentManager import temperament_manager
from .Config import config_parser


__plugin_meta__ = PluginMetadata(
    name="MoEllm聊天",
    description="感谢llm，机器人变聪明了\n✨ 混合专家模型调度LLM插件 | 混合调度·联网搜索·上下文优化·个性定制·Token节约·更加拟人 ✨",
    usage="""1.艾特或以bot的名字开头进行对话\n2.用"性格切换xx"来切换性格（每个性格设定绑定每个人账号，不共享）\n3.用"ai xx"来快速调用纯ai助手\n4.超级管理员限定：用切换模型、切换moe、设置moe、设置联网来设置""",
    type="application",
    homepage="https://github.com/Elflare/nonebot-plugin-moellmchats",
    supported_adapters={"~onebot.v11"},
)

cd = defaultdict(int)
is_repeat_ask_dict = defaultdict(bool)  # 记录是否重复提问

message_matcher = on_message(permission=GROUP, priority=1, block=False)


@message_matcher.handle()
async def context_dict_func(bot: Bot, event: MessageEvent):
    if event.message.extract_plain_text().strip():  # 有文字才记录
        if message_dict := await format_message(event, bot):
            sender_name = event.sender.card or event.sender.nickname
            llm.context_dict[event.group_id].append(
                f"{sender_name}:{''.join(message_dict['text'])}"
            )
        # 概率主动发
        # if random.randint(1, 100) == 1:
        #     llm = llm.MoeLlm(
        # bot, event, message_dict,is_objective=True, temperament='默认')
        #     reply = await llm.handle_llm()


# 性格切换
temperament_switch_matcher = on_command(
    "性格切换", aliases={"切换性格", "人格切换", "切换人格"}, priority=10, block=True
)


@temperament_switch_matcher.handle()
async def _(bot: Bot, event: GroupMessageEvent, args: Message = CommandArg()):
    if temp := args.extract_plain_text().strip():
        if temp in temperament_manager.get_temperaments_keys():
            # 写入文件
            if temperament_manager.set_temperament_dict(event.user_id, temp):
                await temperament_switch_matcher.finish(f"已切换性格为{temp}")
            else:
                await temperament_switch_matcher.finish(
                    "出错了，搞快喊机器人主人来修复一下吧~"
                )
    await temperament_switch_matcher.finish(
        f"只有{temperament_manager.get_temperaments_keys()}中的性格可以切换"
    )


# 查看性格
temperament_check_matcher = on_fullmatch(
    ("查看性格", "查看人格"), priority=10, block=True
)


@temperament_check_matcher.handle()
async def _(event: GroupMessageEvent):
    await temperament_check_matcher.finish(temperament_manager.get_all_temperaments())


check_model_matcher = on_fullmatch("查看模型", priority=10, block=True)


@check_model_matcher.handle()
async def _(event: GroupMessageEvent):
    await check_model_matcher.finish(model_selector.get_model_config())


model_matcher = on_command("切换模型", permission=SUPERUSER, priority=10, block=True)


@model_matcher.handle()
async def _(bot: Bot, event: GroupMessageEvent, args: Message = CommandArg()):
    model_name = args.extract_plain_text().strip()
    result = model_selector.set_chat_model(model_name)
    await model_matcher.finish(result)


set_moe_matcher = on_command("设置moe", permission=SUPERUSER, priority=10, block=True)


@set_moe_matcher.handle()
async def _(bot: Bot, event: GroupMessageEvent, args: Message = CommandArg()):
    is_moe = args.extract_plain_text().strip()
    if is_moe not in ["开", "关", "0", "1"]:
        await model_matcher.finish("参数错误，格式为：设置moe 开、关、1、0")
    if is_moe == "开" or is_moe == "1":
        is_moe = True
    else:
        is_moe = False
    result = model_selector.set_moe(is_moe)
    await model_matcher.finish(result)


set_web_search_matcher = on_command(
    "设置联网", aliases={"切换联网"}, permission=SUPERUSER, priority=10, block=True
)


@set_web_search_matcher.handle()
async def _(bot: Bot, event: GroupMessageEvent, args: Message = CommandArg()):
    is_web_search = args.extract_plain_text().strip()
    if is_web_search not in ["开", "关", "0", "1"]:
        await model_matcher.finish("参数错误，格式为：设置联网 开、关、1、0")
    if is_web_search == "开" or is_web_search == "1":
        is_web_search = True
    else:
        is_web_search = False
    result = model_selector.set_web_search(is_web_search)
    await model_matcher.finish(result)


moe_matcher = on_command("切换moe", permission=SUPERUSER, priority=10, block=True)


@moe_matcher.handle()
async def _(bot: Bot, event: GroupMessageEvent, args: Message = CommandArg()):
    try:
        difficulty, model_name = args.extract_plain_text().split()
        result = model_selector.set_moe_model(model_name, difficulty)
    except Exception:
        await model_matcher.finish("参数错误，格式为：切换moe 难度 模型名")
    await model_matcher.finish(result)


async def handle_llm(
    bot: Bot, event: GroupMessageEvent, matcher, format_message_dict: dict, is_ai=False
):
    # 获取消息文本
    user_id = event.sender.user_id
    if (
        # not model_selector.get_moe()  # 单模型加cd
        # model_selector.get_current_model()["model"] == "deepseek-reasoner"
        event.time - cd[user_id] < config_parser.get_config("cd_seconds")
    ):
        sender_name = event.sender.card or event.sender.nickname
        if is_repeat_ask_dict[user_id]:
            await matcher.finish(
                f"{sender_name}的llm对话cd中, 将会在{config_parser.get_config('cd_seconds') - (event.time-cd[user_id])}秒后自动回答，请不要重复提问~"
            )
        await matcher.send(
            f"{sender_name}的llm对话cd中, 将会在{config_parser.get_config('cd_seconds') - (event.time-cd[user_id])}秒后自动回答，请不要重复提问~"
        )
        is_repeat_ask_dict[user_id] = True
        await asyncio.sleep(
            max(0, config_parser.get_config("cd_seconds") - (event.time - cd[user_id]))
        )
    cd[user_id] = event.time
    if is_ai:
        temp = "ai助手"
    else:
        temp = temperament_manager.get_temperament(user_id)
        if not temp:
            await matcher.finish("出错了，赶快喊机器人主人来修复一下吧~")
    llm_chat = llm.MoeLlm(bot, event, format_message_dict, temperament=temp)
    is_finished = await llm_chat.get_llm_chat()
    is_repeat_ask_dict[user_id] = False  # 重复提问判定就不用了
    if isinstance(is_finished, str):  # 表示失败，失败描述文字
        cd[user_id] = 0
        await matcher.finish(is_finished)
    elif not is_finished:  # 失败后cd回1
        cd[user_id] = 0


llm_matcher = on_message(
    rule=to_me(),
    permission=GROUP,
    priority=99,
    block=True,
)


@llm_matcher.handle()
async def _(bot: Bot, event: MessageEvent):
    if event.message.extract_plain_text().strip():
        format_message_dict = await format_message(event, bot)
    else:
        await llm_matcher.finish(
            Message(random.choice(hello__reply))
        )  # 没有就选一个卖萌回复
    await handle_llm(bot, event, llm_matcher, format_message_dict, is_ai=False)


if config_parser.get_config("fastai_enabled"):
    ai_matcher = on_command(
        "ai",
        permission=GROUP,
        priority=17,
        block=True,
    )

    @ai_matcher.handle()
    async def _(bot: Bot, event: GroupMessageEvent, args: Message = CommandArg()):
        if args.extract_plain_text().strip():
            format_message_dict = await format_message(event, bot)
            await handle_llm(bot, event, ai_matcher, format_message_dict, is_ai=True)
        else:
            await ai_matcher.finish(
                Message(random.choice(hello__reply))
            )  # 没有就选一个卖萌回复


# 优先级10，不会向下阻断，条件：戳一戳bot触发

poke_ = on_notice(rule=to_me(), priority=11, block=False)


@poke_.handle()
async def _poke_event(event: PokeNotifyEvent):
    if event.is_tome:
        await poke_.send(Message(random.choice(poke__reply)))
        # try:
        #     await poke_.send(Message(f"[CQ:group_poke,qq={event.user_id}]"))
        # except ActionFailed:
        #     await poke_.send(Message(f"[CQ:touch,id={event.user_id}]"))
        # except Exception:
        #     return



import nonebot
from nonebot.log import logger
from .Config import config_parser
from random import choice
from pathlib import Path
from os import listdir
from nonebot.adapters.onebot.v11 import MessageSegment
from traceback import format_exc
import re

Bot_NICKNAME: str = list(nonebot.get_driver().config.nickname)[0]  # bot的nickname


# hello之类的回复
hello__reply = [
    "你好喵~",
    "呜喵..？！",
    "你好OvO",
    f"喵呜 ~ ，叫{Bot_NICKNAME}做什么呢☆",
    "怎么啦qwq",
    "呜喵 ~ ，干嘛喵？",
    "呼喵 ~ 叫可爱的咱有什么事嘛OvO",
]

# 戳一戳消息
poke__reply = [
    "嗯？",
    "戳我干嘛qwq",
    "呜喵？",
    "喵！",
    "呜...不要用力戳咱...好疼>_<",
    f"请不要戳{Bot_NICKNAME} >_<",
    "放手啦，不给戳QAQ",
    f"喵 ~ ！ 戳{Bot_NICKNAME}干嘛喵！",
    "戳坏了，你赔！",
    "呜......戳坏了",
    "呜呜......不要乱戳",
    "喵喵喵？OvO",
    "(。´・ω・)ん?",
    "怎么了喵？",
    "呜喵！......不许戳 (,,• ₃ •,,)",
    "有什么吩咐喵？",
    "啊呜 ~ ",
    "呼喵 ~ 叫可爱的咱有什么事嘛OvO",
]


# 表情包解析
def parse_emotion(text: str) -> tuple:
    # 使用正则表达式匹配方括号内的内容
    pattern = r"\[(.*?)\]"
    # 提取所有表情包名字
    names = re.findall(pattern, text)
    # 替换所有匹配项为[表情包名字]
    replaced_text = re.sub(pattern, "", text)
    return replaced_text, names


# 获取表情包名字列表
def get_emotions_names() -> list:
    emotions_names = listdir(config_parser.get_config("emotions_dir"))
    return emotions_names


# 获取具体表情包
def get_emotion(emoji_name: str) -> MessageSegment:
    path = Path(config_parser.get_config("emotions_dir")) / emoji_name
    emotion_image_list = list(path.glob("*"))
    if not emotion_image_list:
        return None
    image = path / choice(emotion_image_list)
    try:
        with open(image, "rb") as f:
            img = f.read()
            return MessageSegment.image(img)
    except OSError:
        logger.warning(format_exc())
        return None


# 消息格式转换
async def format_message(event, bot) -> dict[list, str]:
    text_message = []
    reply_text = ""
    if event.reply:
        reply_text = event.reply.message.extract_plain_text().strip()
        reply = f"[回复 {event.reply.sender.card or event.reply.sender.nickname} 的消息 [{reply_text}]]"
        text_message.append(reply)
    for msgseg in event.get_message():
        if msgseg.type == "at":
            qq = msgseg.data.get("qq")
            if qq != nonebot.get_bot().self_id:  # 排除at机器人
                name = await get_member_name(event.group_id, qq, bot)
                text_message.append(name)
        elif msgseg.type == "image":
            text_message.append("[图片]")
        elif msgseg.type == "face":
            pass
        elif msgseg.type == "text":
            if plain := msgseg.data.get("text", ""):
                if plain.startswith("ai"):  # 判断ai开头
                    text_message.append(plain[2:])
                else:
                    text_message.append(plain)
    return {"text": text_message, "reply": reply_text}


async def get_member_name(group: int, sender_id: int, bot) -> str:  # 将QQ号转换成昵称
    try:
        member_info = await bot.get_group_member_info(
            group_id=group, user_id=sender_id, no_cache=False
        )
        name = member_info.get("card") or member_info.get("nickname")
    except Exception:
        name = sender_id
        logger.warning("获取成员info失败")
    return str(name)



import ujson as json
from pathlib import Path

import nonebot_plugin_localstore as store

config_path: Path = store.get_plugin_config_dir()


class ConfigParser:
    def __init__(self):
        self.filepath = Path(config_path / "config.json")
        self.config = self.parse_config()

    def parse_config(self):
        """Parse the JSON configuration file and return the configuration dictionary."""
        try:
            with open(self.filepath, "r", encoding="utf-8") as file:
                config = json.load(file)
            return config
        except FileNotFoundError:
            # Create a new configuration file with default values
            config = {
                "max_group_history": 10,
                "max_user_history": 8,
                "max_retry_times": 3,
                "user_history_expire_seconds": 600,
                "cd_seconds": 120,
                "search_api": "your api",
                "fastai_enabled": False,
                "emotions_enabled": False,
                "emotion_rate": 0.1,
                "emotions_dir": "absolute path",
            }
            with open(self.filepath, "w", encoding="utf-8") as file:
                json.dump(config, file, indent=4, ensure_ascii=False)
            return config

    def get_config(self, key):
        """Get the value of a configuration item by key."""
        return self.config.get(key)


config_parser = ConfigParser()



from pathlib import Path
import ujson as json

import nonebot_plugin_localstore as store

config_path: Path = store.get_plugin_config_dir()


# 模型选择类
class ModelSelector:
    def __init__(self):
        # 配置文件路径
        self.models_file = Path(config_path / "models.json")
        self.model_config_file = Path(config_path / "model_config.json")

        # 加载配置文件
        self.models = self._load_models()
        self.model_config = self._load_model_config()

        # 初始化缓存

    def _load_models(self):
        # 读取models.json文件，获取多个模型的配置
        if self.models_file.exists():
            with open(self.models_file, "r", encoding="utf-8") as f:
                return json.load(f)
        else:
            # 创建默认
            default_models = {
                "dpsk-chat": {
                    "url": "https://api.deepseek.com/chat/completions",
                    "key": "Bearer xxx",
                    "model": "deepseek-chat",
                    "temperature": 1.5,
                    "max_tokens": 1024,
                    "proxy": "http://127.0.0.1:7890",
                    "stream": True,
                    "is_segment": True,
                    "max_segments": 5,
                },
                "dpsk-r1": {
                    "url": "https://api.deepseek.com/chat/completions",
                    "key": "Bearer xxxx",
                    "model": "deepseek-reasoner",
                    "top_k": 5,
                    "top_p": 1.0,
                },
            }
            self.models_file.parent.mkdir(parents=True, exist_ok=True)
            self.models_file.touch()
            self._write_config(self.models_file, default_models)

    def get_model_config(self):
        return json.dumps(self.model_config, indent=4, ensure_ascii=False)

    def _load_model_config(self):
        # 读取model_config.json文件，获取是否使用MOE及MOE难度模型等配置
        if self.model_config_file.exists():
            with open(self.model_config_file, "r", encoding="utf-8") as f:
                return json.load(f)
        else:
            # 如果model_config.json文件不存在，使用默认配置
            default_config = {
                "use_moe": False,
                "moe_models": {"0": "glm", "1": "glm", "2": "glm"},
                "selected_model": "dpsk-chat",
                "category_model": "glm",
                "use_web_search": False,
            }
            self._write_config(self.model_config_file, default_config)
            return default_config

    def _write_config(self, file_path, config_data):
        # 将配置写入文件
        with open(file_path, "w", encoding="utf-8") as f:
            json.dump(config_data, f, ensure_ascii=False, indent=4)

    def get_moe(self):
        # 获取当前是否使用MOE
        return self.model_config["use_moe"]

    def get_web_search(self):
        # 获取当前是否使用联网
        return self.model_config["use_web_search"]

    def get_model(self, key: str) -> dict:
        # 获取单个模型的配置
        selected_model = self.model_config.get(key)
        if selected_model and selected_model in self.models:
            return self.models[selected_model]

    def get_moe_current_model(self, difficulty: str) -> dict:
        # 获取当前MOE模型的配置
        moe_models = self.model_config["moe_models"]
        model_name = moe_models.get(difficulty)
        if model_name and model_name in self.models:
            # 返回模型的完整配置
            return self.models[model_name]

    def set_moe(self, is_moe: bool = True) -> str:
        # 切换MOE配置
        self.model_config["use_moe"] = is_moe
        self._write_config(self.model_config_file, self.model_config)
        return "已切换为moe" if is_moe else "取消moe"

    def set_web_search(self, is_web_search: bool = True) -> str:
        # 切换联网配置配置
        self.model_config["use_web_search"] = is_web_search
        self._write_config(self.model_config_file, self.model_config)
        return "已开启联网搜索" if is_web_search else "已关闭联网搜索"

    def set_summary_model(self, model_name: str) -> str:
        # 设置单个模型，model_name为models.json中的键
        if model_name not in self.models:
            return f"只能是{list(self.models.keys())}中的模型"

        # 设置selected_model
        self.model_config["summary_model"] = model_name

        # 更新配置文件
        self._write_config(self.model_config_file, self.model_config)
        return f"已切换总结模型为{model_name}的{self.models[model_name]['model']}"

    def set_chat_model(self, model_name: str) -> str:
        # 设置单个模型，model_name为models.json中的键
        if model_name not in self.models:
            return f"只能是{list(self.models.keys())}中的模型"

        # 设置selected_model
        self.model_config["selected_model"] = model_name

        # 更新配置文件
        self._write_config(self.model_config_file, self.model_config)
        return f"已切换聊天模型为{model_name}的{self.models[model_name]['model']}"

    def set_moe_model(self, model_name: str, difficulty: str) -> str:
        # 设置MOE模型，model_name为models.json中的键，difficulty为0、1或2
        if model_name not in self.models:
            return f"只能是{list(self.models.keys())}中的模型"

        if difficulty not in ["0", "1", "2"]:
            return "difficulty只能是0、1、2中的一个"

        # 更新MOE模型配置
        self.model_config["moe_models"][difficulty] = model_name

        # 更新配置文件
        self._write_config(self.model_config_file, self.model_config)
        return f"已将{difficulty}的模型切换为{model_name}的{self.models[model_name]['model']}"


model_selector = ModelSelector()



import ujson as json
from traceback import format_exc
from nonebot.log import logger
from pathlib import Path

import nonebot_plugin_localstore as store

config_path: Path = store.get_plugin_config_dir()


# 性格设置类
class TemperamentManager:
    def __init__(self):
        self.temperament_config = Path(
            config_path / "temperament_config.json"
        )
        self.temperaments_path = Path(config_path / "temperaments.json")
        self.temperaments = self.read_temperaments()
        self.temperament_dict = self.read_temperament()

    def get_temperament(self, qq=None) -> str:
        """根据qq获取每个群友的性格配置"""
        if qq:
            qq = str(qq)
            return self.temperament_dict.get(qq, "默认")
        return "默认"

    def get_temperaments_keys(self) -> list:
        return self.temperaments.keys()

    def get_all_temperaments(self) -> str:
        return json.dumps(self.temperaments, indent=4, ensure_ascii=False)

    def get_temperament_prompt(self, temperament: str) -> str:
        """根据性格获取提示语"""
        return self.temperaments.get(temperament, "你是ai助手。回答像真人且简短")

    def set_temperament_dict(self, qq, temperament) -> bool:
        """设置配置项的值"""
        qq = str(qq)
        self.temperament_dict[qq] = temperament
        return self.write_temperament(qq, temperament)

    # 读取文件
    def read_temperament(self) -> str:
        if not self.temperament_config.exists():
            self.temperament_config.parent.mkdir(parents=True, exist_ok=True)
            self.temperament_config.touch()
            with open(self.temperament_config, "w", encoding="utf-8") as f:
                json.dump({}, f, ensure_ascii=False, indent=4)
            return {}
        try:
            with open(self.temperament_config, "r", encoding="utf-8") as f:
                return json.load(f)
        except Exception:
            logger.error(format_exc())
            return {}

    # 读取文件
    def read_temperaments(self) -> str:
        prompt = "你是ai助手。回答像真人且简短"
        if not self.temperaments_path.exists():
            self.temperaments_path.parent.mkdir(parents=True, exist_ok=True)
            self.temperaments_path.touch()
            with open(self.temperaments_path, "w", encoding="utf-8") as f:
                json.dump({"默认": prompt}, f, ensure_ascii=False, indent=4)
            return prompt
        try:
            with open(self.temperaments_path, "r", encoding="utf-8") as f:
                return json.load(f)
        except Exception:
            logger.error(format_exc())
            return prompt

    # 性格写入文件
    def write_temperament(self, qq: int, temperament: str) -> bool:
        if not self.temperament_config.exists():
            self.temperament_config.parent.mkdir(parents=True, exist_ok=True)

            self.temperament_config.touch()
        try:
            with open(self.temperament_config, "r+", encoding="utf-8") as f:
                if data := f.read():
                    dict_ = json.loads(data)
                    dict_[qq] = temperament
                else:
                    dict_ = {qq: temperament}
                f.seek(0)
                json.dump(dict_, f, ensure_ascii=False, indent=4)
                f.truncate()
                return True
        except Exception:
            logger.error(format_exc())
            return False


temperament_manager = TemperamentManager()



from collections import defaultdict, deque
import time
from .Config import config_parser

messages_dict = defaultdict(
    lambda: deque(maxlen=config_parser.get_config("max_user_history"))
)  # {'qq':messages_entity_list}
# messages_entity_list = [messages_entity1, messages_entity2]


class MessagesEntity:
    def __init__(self, timestamp):
        self.timestamp = timestamp
        self.user_msg = None
        self.assistant_msg = None

    def add_user_msg(self, user_msg: dict):
        self.user_msg = user_msg

    def add_assistant_msg(self, assistant_msg: dict):
        self.assistant_msg = assistant_msg

    def get_user_msg(self) -> dict:
        return self.user_msg

    def get_assistant_msg(self) -> dict:
        return self.assistant_msg


class MessagesHandler:
    def __init__(self, user_id):
        self.user_id = user_id
        self.timestamp = time.time()
        self.messages_entity = MessagesEntity(self.timestamp)
        self.messages_entity_list = messages_dict[self.user_id]

    def clrear_messages(self):
        self.messages_entity_list = []

    # 预处理用户问题
    def pre_process(self, format_message_dict: dict) -> str:
        if self.messages_entity_list:  # 之前有对话
            # 超过时间一对对话的删了
            for i in range(len(self.messages_entity_list) - 1, -1, -1):
                messages_entity = self.messages_entity_list[i]
                if time.time() - messages_entity.timestamp > config_parser.get_config(
                    "user_history_expire_seconds"
                ):
                    self.messages_entity_list.popleft()
            if (
                self.messages_entity_list  # 还有对话
                and format_message_dict["reply"]  # 有回复
                and format_message_dict["reply"].strip()
                == self.messages_entity_list[-1]
                .get_assistant_msg()["content"]
                .strip()  # 如果引用的就是上一条回复
            ):
                format_message_dict["text"].pop(0)

        plain = "".join(format_message_dict["text"])
        self.new_user_msg = {"role": "user", "content": plain}  # 最新的问题
        self.messages_entity.add_user_msg(
            self.new_user_msg
        )  # 添加用户问题，之后再处理回答
        return plain

    def search_message_handler(self, search_info: str):
        # 添加搜索信息
        self.new_user_msg["content"] += (
            f"\n(这是联网搜索结果，供你进行参考： {search_info})"
        )

    def append_message_list(self, messages_entity):
        messages_dict[self.user_id].append(self.messages_entity)

    def get_send_message_list(self) -> list:
        result = []
        for messages_entity in self.messages_entity_list:
            # 成对添加
            result.append(messages_entity.get_user_msg())
            result.append(messages_entity.get_assistant_msg())
        result.append(self.messages_entity.get_user_msg())
        return result

    # 后处理
    def post_process(self, assistant_msg: str = None):
        if assistant_msg:
            self.messages_entity.add_assistant_msg(
                {"role": "assistant", "content": assistant_msg}
            )
            messages_dict[self.user_id].append(self.messages_entity)



import ujson as json
from nonebot.log import logger
import traceback
import aiohttp
from .ModelSelector import model_selector


class Categorize:
    def __init__(self, plain):
        self.plain = plain

    async def get_category(self) -> tuple[int, bool]:
        prompt = """你是一个问题分类器。当我给你一句话时，你的任务是根据问题的难度对其进行分类，并判断是否需要互联网连接来回答。你永远不回答该问题，只需返回一个 JSON 结构（不带其他格式），如下：

{
  "difficulty": "0 | 1 | 2",
  "internet_required": true | false,
  "key_word": string
}
说明：

difficulty: 用来表示问题的难度等级：
"0": 简单直接明了的问题，几乎无需思考或计算。
"1": 中等难度的问题，可能需要一定的思考、分析或计算。
"2": 高难度的问题，需要深厚的专业知识或广泛的研究才能回答。
internet_required: 布尔值，表示是否需要访问互联网或外部数据库来提供完整答案：true: 需要访问外部信息或数据库。false: 不需要互联网连接，可以仅凭知识回答。
"key_word": 字符串。当internet_required为true时，根据这句话的内容，判断哪些是用来搜索的字符串，用空格分隔。若为false，则返回空。
"""

        send_message_list = [
            {"role": "system", "content": prompt},
            {"role": "user", "content": self.plain},
        ]
        data = {
            "model": model_selector.get_model("category_model")["model"],
            "messages": send_message_list,
            "temperature": 0,
        }

        data = json.dumps(data)
        headers = {
            "Authorization": model_selector.get_model("category_model")["key"],
            "Content-Type": "application/json",
        }
        for try_times in range(2):
            try:
                if try_times > 0:  # 说明失败了，再来一次
                    self.plain += "\n(注意不是直接回答以上内容，且上述所有内容仅需要进行一次分类和判断联网，回复我的格式为json，不需要任何其他内容)"
                async with aiohttp.ClientSession() as session:
                    async with session.post(
                        url=model_selector.get_model("category_model")["url"],
                        data=data,
                        headers=headers,
                        timeout=300,
                        proxy=model_selector.get_model("category_model").get("proxy"),
                    ) as resp:
                        response = await resp.json()
                if choices := response.get("choices"):
                    result = choices[0]["message"]["content"]
                    result = json.loads(result)
                    return (
                        str(result["difficulty"]),
                        result["internet_required"],
                        result["key_word"],
                    )
                elif (
                    response.get("code") == "DataInspectionFailed"
                    or 'contentFilter' in response
                ):
                    logger.warning(response)
                    return "内容不合规，拒绝回答"
            except Exception:
                logger.warning(traceback.format_exc())
                continue
        return False



import ujson as json
import aiohttp
import asyncio
import traceback
from asyncio import TimeoutError
from nonebot.log import logger
from collections import defaultdict, deque
from .Categorize import Categorize
from .Search import Search
from .ModelSelector import model_selector
from .MessagesHandler import MessagesHandler
from .Config import config_parser
from .TemperamentManager import temperament_manager
from .utils import get_emotions_names, get_emotion, parse_emotion
import random

context_dict = defaultdict(
    lambda: deque(maxlen=config_parser.get_config("max_group_history"))
)


class MoeLlm:
    def __init__(
        self,
        bot,
        event,
        format_message_dict: dict,
        is_objective: bool = False,
        temperament="默认",
    ):
        self.bot = bot
        self.event = event
        self.format_message_dict = format_message_dict
        self.user_id = event.user_id
        self.is_objective = is_objective
        self.temperament = temperament
        self.model_info = None
        self.emotion_flag = False  # 判断本次对话是否发送表情包
        self.prompt = f"{temperament_manager.get_temperament_prompt(temperament)}。我的id是{ event.sender.card or event.sender.nickname}"

    async def send_emotion_message(self, content: str) -> str:
        """处理和发送表情包
        Returns: str: 替换表情之后的内容
        """
        if self.emotion_flag:  # 本次对话发送表情包
            content, emotion_names_list = parse_emotion(content)
            if content:
                await self.bot.send(self.event, content)
            for emotion_name in emotion_names_list:
                # 发送
                if emotion := get_emotion(emotion_name):
                    await self.bot.send(self.event, emotion)
        else:  # 默认直接发送
            await self.bot.send(self.event, content)
        return content

    async def stream_llm_chat(
        self, session, url, headers, data, proxy, is_segment=False
    ) -> bool:
        # 流式响应内容
        buffer = []
        assistant_result = []  # 后处理助手回复
        punctuation_buffer = ""  # 存标点
        is_second_send = False  # 不是第一次发送
        async with session.post(
            url, headers=headers, json=data, proxy=proxy
        ) as response:
            # 确保响应是成功的
            if response.status == 200:
                # 异步迭代响应内容
                MAX_SEGMENTS = self.model_info.get("max_segments", 5)
                current_segment = 0
                jump_out = False  # 判断是否跳出循环
                async for line in response.content:
                    if (
                        not line
                        or line.startswith(b"data: [DONE]")
                        or line.startswith(b"[DONE]")
                        or jump_out
                    ):
                        break  # 结束标记，退出循环e.content:
                    if line.startswith(b"data:"):
                        decoded = line[5:].decode("utf-8")
                    elif line.startswith(b""):
                        decoded = line.decode("utf-8")
                    if not decoded.strip():
                        continue
                    json_data = json.loads(decoded)
                    if message := json_data.get("choices", [{}])[0].get("message", {}):
                        content = message.get("content", "")
                    elif message := json_data.get("choices", [{}])[0].get("delta", {}):
                        content = message.get("content", "")
                    if content:
                        if is_segment and self.temperament != "ai助手":  # 分段
                            for char in content:
                                if char in ["。", "？", "！", "—", "\n"]:
                                    punctuation_buffer += char
                                else:
                                    if punctuation_buffer:
                                        # 发送累积的标点内容
                                        current_content = (
                                            "".join(buffer) + punctuation_buffer
                                        ).strip()
                                        if current_content.strip():
                                            if current_segment >= MAX_SEGMENTS:
                                                TOO_LANG = "太长了，不发了"
                                                buffer = [TOO_LANG]
                                                jump_out = True
                                                break
                                            if (
                                                is_second_send
                                            ):  # 第二次开始，会等几秒再发送
                                                await asyncio.sleep(
                                                    2 + len(current_content) / 3
                                                )
                                            else:
                                                is_second_send = True
                                            # 处理表情包和发送
                                            current_content = (
                                                await self.send_emotion_message(
                                                    current_content
                                                )
                                            )
                                            current_segment += 1
                                            assistant_result.append(current_content)
                                        buffer = []
                                        punctuation_buffer = ""
                                    buffer.append(char)
                        else:
                            buffer.append(content)
                # 最后的的句子或者没分段
                if jump_out:
                    result = "".join(buffer)
                else:
                    result = "".join(buffer) + punctuation_buffer
                if is_second_send:
                    await asyncio.sleep(2 + len(current_content) / 3)
                else:
                    is_second_send = True
                if result := result.strip():
                    result = await self.send_emotion_message(result)
                    if not self.is_objective:
                        self.messages_handler.post_process(
                            "".join(assistant_result) + result
                        )
                    return True
                elif is_second_send:
                    if not self.is_objective:
                        self.messages_handler.post_process("".join(assistant_result))
                    return True  # 前面有，最后一句没回复，也当回完了
            else:
                logger.error(f"Error: {response}")
        return False

    async def none_stream_llm_chat(self, session, url, headers, data, proxy) -> bool:
        async with session.post(
            url=url,
            data=data,
            headers=headers,
            ssl=False,
            proxy=proxy,
        ) as resp:
            # 获取整个响应文本
            response = await resp.json()
            # 返回200
            if resp.status != 200 or not response:
                logger.error(response)
                return False
        if choices := response.get("choices"):
            content = choices[0]["message"]["content"]
            start_tag = "<think>"
            end_tag = "</think>"
            start = content.find(start_tag)
            end = content.find(end_tag)
            if start == -1 and end != -1:
                end += len(end_tag)
                start = 0
                result = content[:start] + content[end:]
            elif start != -1 and end != -1:
                end += len(end_tag)
                result = content[:start] + content[end:]
            else:
                result = content
        else:
            logger.error(response)
            return False
        if not self.is_objective:
            self.messages_handler.post_process(result.strip())
        await self.bot.send(self.event, result.strip())
        return True

    def prompt_handler(self):
        """处理system prompt，表情包和上下文相关"""
        if self.temperament != "ai助手":  # 不为ai助手才加上下文
            # 表情包
            if (
                config_parser.get_config("emotions_enabled")
                and self.model_info.get("is_segment")
                and self.model_info.get("stream")
                and random.random() < config_parser.get_config("emotion_rate")
            ):
                self.emotion_flag = True
                emotion_prompt = f"。回复时根据回答内容，发送表情包，每次回复最多发一个表情包，格式为中括号+表情包名字，如：[表情包名字]。可选表情有{get_emotions_names()}"
            else:
                emotion_prompt = ""
            self.prompt += f"。现在你在一个qq群中,你只需回复我{emotion_prompt}。群里近期聊天内容，冒号前面是id，后面是内容：\n"
            # 去除群聊最新的对话，因为在用户的上下文中
            context_dict_ = list(context_dict[self.event.group_id])[:-1]
            self.prompt += "\n".join(context_dict_)

    async def get_llm_chat(self) -> str:
        self.messages_handler = MessagesHandler(self.user_id)
        plain = self.messages_handler.pre_process(self.format_message_dict)
        # 获取难度和是否联网
        if model_selector.get_moe() or model_selector.get_web_search():
            category = Categorize(plain)
            category_result = await category.get_category()
            if isinstance(category_result, str):  # 如果是str，则拒绝回答
                return category_result
            if isinstance(category_result, tuple):  # 如果是tuple，则说明没有问题
                difficulty, internet_required, key_word = category_result
                logger.info(
                    f"难度：{difficulty}, 是否联网：{internet_required}，搜索关键词：{key_word}"
                )
                # 判断是否联网
                if internet_required and model_selector.get_web_search():
                    search = Search(key_word)
                    await self.bot.send(self.event, "検索中...検索中...=￣ω￣=")
                    if search_result := await search.get_search():
                        self.messages_handler.search_message_handler(search_result)
                    elif isinstance(search_result, bool):
                        await self.bot.send(self.event, "没搜到，可能没有相关内容")
                    else:
                        await self.bot.send(
                            self.event, "搜索失败，请检查日志输出"
                        )  # 搜索失败
                # 根据难度改key和url
                if model_selector.get_moe():  # moe
                    self.model_info = model_selector.get_moe_current_model(difficulty)
        if not self.model_info:  # 分类失败或者不是用的moe
            self.model_info = model_selector.get_model("selected_model")
        logger.info(f"模型选择为：{self.model_info['model']}")
        # 处理system prompt，表情包和上下文相关
        self.prompt_handler()
        send_message_list = self.messages_handler.get_send_message_list()
        send_message_list.insert(0, {"role": "system", "content": self.prompt})
        data = {
            "model": self.model_info["model"],
            "reasoning_effort": "none",
            "messages": send_message_list,
            "max_tokens": self.model_info.get("max_tokens"),
            "temperature": self.model_info.get("temperature"),
            "top_p": self.model_info.get("top_p"),
            "stream": self.model_info.get("stream", False),
            # "tools": [
            #     {
            #         "type": "web_search",
            #         "web_search": {"enable": True},
            #     }
            # ],
        }
        # 有的模型没有top_k
        if self.model_info.get("top_k"):
            data["top_k"] = self.model_info.get("top_k")

        headers = {
            "Authorization": self.model_info["key"],
            "Content-Type": "application/json",
        }
        async with aiohttp.ClientSession(
            timeout=aiohttp.ClientTimeout(total=300)
        ) as session:
            max_retry_times = (
                config_parser.get_config("max_retry_times")
                if config_parser.get_config("max_retry_times")
                else 3
            )
            result = ""
            for retry_times in range(max_retry_times):
                if retry_times > 0:
                    await self.bot.send(
                        self.event,
                        f"api又卡了呐！第 {retry_times+1} 次尝试，请勿多次发送~",
                    )
                    await asyncio.sleep(2**retry_times)
                try:
                    if self.model_info.get("stream"):
                        result = await self.stream_llm_chat(
                            session,
                            self.model_info["url"],
                            headers,
                            data,
                            self.model_info.get("proxy"),
                            self.model_info.get("is_segment"),
                        )
                    else:
                        data = json.dumps(data)
                        result = await self.none_stream_llm_chat(
                            session,
                            self.model_info["url"],
                            headers,
                            data,
                            self.model_info.get("proxy"),
                        )
                    if result:
                        return result  # 正常返回从这里
                    else:  # 出错
                        continue
                except TimeoutError:
                    return "网络超时呐，多半是api反应太慢（"
                except Exception:
                    logger.warning(str(send_message_list))
                    traceback.print_exc()
                    continue
            return "api寄！"
